//  ------------------------------------------------------------------------
//
//  Copyright (C) 2011-2012 Fons Adriaensen <fons@linuxaudio.org>
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//  ------------------------------------------------------------------------


#ifndef __GLOBAL_H
#define __GLOBAL_H


enum { MAXFILT = 3, MAXBAND = MAXFILT + 1, MAXCHAN = 16, PERIOD = 64 };
enum { EV_GO_PROCESS = 24, EV_GO_SILENCE, EV_GO_BYPASS, EV_SHUTDOWN = 31 };


#endif
